# copyright (C) 1997-2000 Jean-Luc Fontaine (mailto:jfontain@free.fr)
# this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu

set rcsId {$Id: modgui.tcl,v 1.1.1.6 2000/06/12 10:40:21 jfontain Exp $}


class moduleOperations {

    proc moduleOperations {this action {parentPath .}} {                                    ;# action can be display, load or unload
        if {[string compare $action display]==0} {
            set buttons o
        } else {
            set buttons oc
        }
        set dialog [new dialogBox $parentPath\
            -buttons $buttons -default o -die 0 -command "moduleOperations::done $this"\
            -x [winfo pointerx $parentPath] -y [winfo pointery $parentPath] -deletecommand "delete $this"\
        ]
        set frame [frame $widget::($dialog,path).frame]
        grid columnconfigure $frame 1 -weight 1
        set moduleOperations::($this,label) [label $frame.label -font $font::(mediumNormal)]
        grid $moduleOperations::($this,label) -row 0 -column 0 -sticky nw
        set entry [new comboEntry $frame -font $font::(mediumBold)]
        composite::configure $entry button scroll -height 5
        grid $widget::($entry,path) -row 0 -column 1 -sticky new
        set scroller [new scroller $frame -height 100 -width 200]
        set moduleOperations::($this,container) [frame $widget::($scroller,path).container]
        scroller::display $scroller $moduleOperations::($this,container)
        grid rowconfigure $frame 1 -weight 1
        grid $widget::($scroller,path) -row 1 -column 0 -columnspan 2 -sticky nsew
        dialogBox::display $dialog $frame
        set moduleOperations::($this,dialog) $dialog
        set moduleOperations::($this,frame) $frame
        set moduleOperations::($this,entry) $entry
        set moduleOperations::($this,scroller) $scroller
        set moduleOperations::($this,action) $action
        switch $action {
            display {
                composite::configure $dialog -title {moodss: Loaded modules}
                loaded $this 0
            }
            load {
                composite::configure $dialog -title {moodss: Load module}
                discover $this
            }
            unload {
                composite::configure $dialog -title {moodss: Unload module}
                loaded $this 1
            }
        }
        composite::configure $entry -editable 0                                      ;# user is not allowed to input an invalid name
        set moduleOperations::($this,module) {}                                                                   ;# selected module
    }

    proc ~moduleOperations {this} {
        delete $moduleOperations::($this,scroller)                                           ;# delete objects other than dialog box
    }

    proc discover {this} {
        if {![info exists moduleOperations::(discoveredModules)]} {                              ;# cache modules and their switches
            $moduleOperations::($this,label) configure -text {Searching for modules... Found:}
            $moduleOperations::($this,frame) configure -cursor watch                                   ;# show user that we are busy
            update idletasks
            set path $composite::($moduleOperations::($this,entry),entry,path)
            modules::available "
                lappend moduleOperations::(discoveredModules) %M
                set moduleOperations::(%M,discoveredSwitches) %S                                             ;# save module switches
                $path delete 0 end
                $path insert 0 %M                                                       ;# show modules one by one for user feedback
                update idletasks
            "
            $moduleOperations::($this,frame) configure -cursor {}
            $path delete 0 end
        }
        $moduleOperations::($this,label) configure -text {Select available module:}
        update idletasks
        set modules [lsort $moduleOperations::(discoveredModules)]                               ;# sort modules for easy navigation
        foreach module $modules {
            foreach {option argument} [set moduleOperations::($module,switches) $moduleOperations::($module,discoveredSwitches)] {
                if {$argument} {
                    set moduleOperations::($this,$module,$option) {}
                } else {
                    set moduleOperations::($this,$module,$option) 0
                }
            }
        }
        composite::configure $moduleOperations::($this,entry) -list $modules -command "moduleOperations::selection $this"
    }

    proc loaded {this unload} {
        $moduleOperations::($this,label) configure -text {Select loaded module:}
        foreach {namespace options} [modules::loaded] {
            lappend namespaces $namespace
            set switches {}
            foreach {switch argument value} $options {
                lappend switches $switch $argument
                set moduleOperations::($this,$namespace,$switch) $value
            }
            set moduleOperations::($namespace,switches) $switches
        }
        if {[info exists namespaces]} {
            if {$unload} {
                $moduleOperations::($this,label) configure -text {Select module to unload:}
            } else {
                $moduleOperations::($this,label) configure -text {Select loaded module:}
            }
            composite::configure $moduleOperations::($this,entry) -list $namespaces -command "moduleOperations::selection $this"
        } else {
            $moduleOperations::($this,label) configure -text {No loaded modules:}
            composite::configure $moduleOperations::($this,entry) -state disabled
        }
    }

    proc selection {this module} {
        if {[string compare $module $moduleOperations::($this,module)]==0} return
        set frame $moduleOperations::($this,container)
        eval destroy [grid slaves $frame]                                                  ;# first remove existing switches display
        grid columnconfigure $frame 1 -weight 1
        if {[string compare $($this,action) display]==0} {                                            ;# set parameter widgets state
            set state disabled
        } else {
            set state normal
        }
        set row 0
        foreach {option argument} $moduleOperations::($module,switches) {
            grid [label $frame.$row,0 -font $font::(mediumBold) -text $option] -padx 5 -sticky w -row $row -column 0
            if {$argument} {
                set path [entry $frame.$row,1\
                    -font $font::(mediumNormal) -width 40 -textvariable moduleOperations::($this,$module,$option) -state $state\
                ]
            } else {
                set path [checkbutton $frame.$row,1 -variable moduleOperations::($this,$module,$option) -state $state]
            }
            grid $path -sticky ew -row $row -column 1
            incr row
        }
        if {$row==0} {                                                                                 ;# no options for this module
            grid [label $frame.$row,0 -font $font::(mediumItalic) -text {no options}] -padx 5 -sticky w -row $row -column 0
        }
        set moduleOperations::($this,module) $module
    }

    proc done {this} {
        set delete 1
        set module $moduleOperations::($this,module)
        if {[string length $module]>0} {
            switch $moduleOperations::($this,action) {
                load {
                    set string $module
                    foreach {option argument} $moduleOperations::($module,switches) {
                        if {$argument} {
                            if {[string length $moduleOperations::($this,$module,$option)]>0} {
                                append string " $option $moduleOperations::($this,$module,$option)"
                            }
                        } else {
                            if {$moduleOperations::($this,$module,$option)} {
                                append string " $option"
                            }
                        }
                    }
                    if {[catch {dynamicallyLoadModules $string} message]} {
                        tk_messageBox -title {moodss: Error loading module} -type ok -icon error -message $message
                        set delete 0
                    }
                }
                unload {
                    dynamicallyUnloadModule $module
                }
            }
        }
        if {$delete} {
            delete $moduleOperations::($this,dialog)                                            ;# which in turn deletes this object
        }
    }

}
