# copyright (C) 1997-2000 Jean-Luc Fontaine (mailto:jfontain@free.fr)
# this program is free software: please read the COPYRIGHT file enclosed in this package or use the Help Copyright menu

set rcsId {$Id: scrollbl.tcl,v 1.2.1.3 2000/04/24 10:49:59 jfontain Exp $}


class scrollingLabel {

    proc scrollingLabel {this parentPath args} composite {[new frame $parentPath] $args} {
        composite::manage $this [new label $widget::($this,path) -font $widget::(default,ButtonFont)] label
        composite::complete $this
        bind $widget::($this,path) <Configure> "scrollingLabel::refresh $this %w"     ;# now that everything is initialized properly
    }

    proc ~scrollingLabel {this} {}

    proc options {this} {                                                                               ;# force text initialization
        return [list\
            [list -font font Font $widget::(default,ButtonFont) $widget::(default,ButtonFont)]\
            [list -interval interval Interval 15 15]\
            [list -step step Step 1]\
            [list -text text Text {}]\
            [list -width width Width 0 0]\
        ]
    }

    proc set-interval {this value} {}

    proc set-step {this value} {
        set scrollingLabel::($this,step) -$value                                                                ;# start moving left
    }

    proc set-font {this value} {
        $composite::($this,label,path) configure -font $value
        refresh $this [winfo width $widget::($this,path)]                                                           ;# force refresh
    }

    proc set-text {this value} {
        $composite::($this,label,path) configure -text $value
        refresh $this [winfo width $widget::($this,path)]                                                           ;# force refresh
    }

    proc set-width {this value} {
        $widget::($this,path) configure -width $value
    }

    proc refresh {this width} {
        if {![info exists scrollingLabel::($this,step)]} return                                               ;# not initialized yet
        set scrollingLabel::($this,textWidth) [winfo reqwidth $composite::($this,label,path)]
        $widget::($this,path) configure -height [winfo reqheight $composite::($this,label,path)]
        place $composite::($this,label,path) -anchor nw -x [set scrollingLabel::($this,x) 0] -y 0
        if {$width<$scrollingLabel::($this,textWidth)} {                                                       ;# scrolling required
            scroll $this $width
        } else {
            catch {after cancel $scrollingLabel::($this,event)}
        }
    }

    proc scroll {this width} {
        set interval $composite::($this,-interval)
        if {\
            (($scrollingLabel::($this,step)<0)&&($scrollingLabel::($this,x)<($width-$scrollingLabel::($this,textWidth))))||\
            (($scrollingLabel::($this,step)>0)&&($scrollingLabel::($this,x)>0))\
        } {
            set scrollingLabel::($this,step) [expr {-$scrollingLabel::($this,step)}]                             ;# change direction
            set interval [expr {20*$interval}]                                                                      ;# pause a while
        }
        place $composite::($this,label,path) -x [incr scrollingLabel::($this,x) $scrollingLabel::($this,step)]
        catch {after cancel $scrollingLabel::($this,event)}                                                 ;# avoid race conditions
        set scrollingLabel::($this,event) [after $interval "scrollingLabel::scroll $this $width"]
    }

}
